{% load i18n %}
{% load static %}

<table class="table table-dark table-striped table-hover w-100" id="skill-list-table">
    <thead>
        <th class="col-name w-auto">{% trans "Name" %}</th>
        <th class="col-skills w-auto">{% trans "Skills" %}</th>
        <th class="col-active w-auto">{% trans "Active" %}</th>
        <th class="col-ordering w-auto">{% trans "Order-Weight" %}</th>
        <th class="col-category w-auto">{% trans "Category" %}</th>
        <th class="col-actions w-auto">{% trans "Actions" %}</th>
    </thead>

    <tbody>
    </tbody>
</table>
