{% extends 'madc/base.html' %}
{% load i18n %}
{% load humanize %}
{% load bootstrap %}

{% block page_title %}
    {{ title }} ⋗ {% translate "Doctrine Checker" %}
{% endblock %}

{% block page_topic %}
    <h1 class="page-header text-center">
        {{ title }}
    </h1>
{% endblock page_topic %}

{% block view_block %}
<div class="card-body bg-primary rounded-top d-flex align-items-center">
    <h3 class="me-3">{% translate "Doctrine Checker" %}</h3>

</div>
<div class="card-body rounded-bottom">
    <div class="mb-3">
        <div id="category-filter" class="d-flex flex-wrap gap-2">
            <div class="form-check">
                <input class="form-check-input" type="checkbox" value="" id="all-categories" checked>
                <label class="form-check-label" for="all-categories">
                    {% translate "All Categories" %}
                </label>
            </div>
        </div>
    </div>
    {% include 'madc/partials/table/skill-checker.html' %}
</div>
{% endblock %}

{% block extra_javascript %}
    {% include 'bundles/datatables-css-bs5.html' %}
    {% include 'bundles/datatables-js-bs5.html' %}
    {% include "bundles/filterdropdown-js.html" %}
    {% include 'bundles/clipboard-js.html' %}
    {% include 'madc/bundles/settings-bundles.html' %}
    {% include 'madc/bundles/skill-checker-bundles.html' %}

    {% include 'madc/partials/modals/modal.html' with name="ViewMissing" %}

    <script type="application/javascript">
        setupModal('#modalViewMissingContainer', 'ajax_missing', '#modalViewMissingContent', '#modalViewMissingLoader');
    </script>
{% endblock %}

{% block extra_css %}
{% endblock %}

{% block extra_script %}
{% endblock %}
