{% load i18n %}
{% load static %}

<table class="table table-dark table-striped table-hover w-100" id="skill-checker-table">
    <thead>
        <th class="col-main-character w-auto">{% trans "Account" %}</th>
        <th class="col-total-amount w-auto">{% trans "Doctrines" %}</th>
    </thead>

    <tbody>

    </tbody>
</table>
