# Generated by Django 4.2.11 on 2025-07-04 10:29

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("madc", "0002_alter_general_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="skilllist",
            options={"default_permissions": ()},
        ),
    ]
